/*****************************************************************

Copyright (c) 1996-2003 the kicker authors. See file AUTHORS.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

#include <qapplication.h>
#include <qpainter.h>
#include <qstyle.h>

#include <kstandarddirs.h>
#include <kglobal.h>
#include <kmimetype.h>
#include <kglobalsettings.h>
#include <kcursor.h>
#include <kapplication.h>
#include <kipc.h>
#include <kdebug.h>

#include "panel.h"
#include "zoombutton.h"
#include "containerarea.h"

#include "panelbuttonbase.h"
#include "panelbuttonbase.moc"

namespace {
    ZoomButton* zoomButton = 0;
    int zoomDisabled = 0;
}

PanelButtonBase::PanelButtonBase(QWidget *parent, const char *name, WFlags f)
  : QButton(parent, name, f)
  , _valid(true)
  , _drawArrow(false)
  , _highlight(false)
  , _animated(false)
  , _changeCursorOverItem(true)
  , _hasAcceptedDrag(false)
  , _hasAnimation(true)
  , _wasTransparent(false)
  , _tile(QString::null)
  , _iconName(QString::null)
  , _bgDirty(true)
  , _movie(0L)
  , _arrowDirection(::Bottom)
  , _popupDirection(::dUp)
  , _orientation(Horizontal)
  , _size((KIcon::StdSizes)-1), _zoom_size((KIcon::StdSizes)-1)
{
    setBackgroundOrigin( WindowOrigin );
    calculateIconSizes();

    slotSettingsChanged(KApplication::SETTINGS_MOUSE);
    connect(kapp, SIGNAL(settingsChanged(int)), SLOT(slotSettingsChanged(int)));
    kapp->addKipcEventMask(KIPC::SettingsChanged);

    setAcceptDrops(true);
}

void PanelButtonBase::configure()
{
    _bgDirty = true;
}

void PanelButtonBase::slotSettingsChanged(int category)
{
    if (category != KApplication::SETTINGS_MOUSE) return;

    _changeCursorOverItem = KGlobalSettings::changeCursorOverIcon();

    if (_changeCursorOverItem)
	setCursor(KCursor::handCursor());
    else
	unsetCursor();
}

void PanelButtonBase::setTile(const QString& tile)
{
    if( tile != _tile ) {
        _tile = tile;
        loadTiles();
        update();
    }
}

void PanelButtonBase::setIcon(const QString& icon)
{
    if( icon != _iconName ) {
        _iconName = icon;
        loadIcons();
        update();
	emit iconChanged();
    }
}

// (re)calculates the icon sizes and report true if they have changed.
bool PanelButtonBase::calculateIconSizes()
{
    KIcon::StdSizes sz;
    KIcon::StdSizes zoom_sz;

    if ( orientation() == Horizontal ) {
        if ( height() < 32 ) sz = KIcon::SizeSmall;
        else if ( height() < 48 ) sz = KIcon::SizeMedium;
        else sz = KIcon::SizeLarge;
    }
    else {
        if ( width() < 32 ) sz = KIcon::SizeSmall;
        else if ( width() < 48 ) sz = KIcon::SizeMedium;
        else sz = KIcon::SizeLarge;
    }

    if (sz == KIcon::SizeSmall)
        zoom_sz = KIcon::SizeMedium;
    else
        zoom_sz = KIcon::SizeLarge;

    if (_size != sz || _zoom_size != zoom_sz) {
        _size = sz;
        _zoom_size = zoom_sz;
	return true;
     }
     return false;
}

void PanelButtonBase::loadIcons()
{
    KIconLoader * ldr = KGlobal::iconLoader();
    QString nm = _iconName;

    _icon = ldr->loadIcon(nm, KIcon::Panel, _size, KIcon::DefaultState, 0L, true);

    if (_icon.isNull()) {
        nm = defaultIcon();
	_icon = ldr->loadIcon(nm, KIcon::Panel, _size, KIcon::DefaultState);
    }

    _iconh = ldr->loadIcon(nm, KIcon::Panel, _size, KIcon::ActiveState, 0L, true);
    _iconz = ldr->loadIcon(nm, KIcon::Panel, _zoom_size, KIcon::ActiveState, 0L, true );
}

void PanelButtonBase::setIconURL(const KURL & u)
{
    QString name = KMimeType::iconForURL(u, 0);
    setIcon( name );
}

void PanelButtonBase::setTitle(const QString & t)
{
    _title = t;
}

void PanelButtonBase::setDrawArrow(bool v)
{
    if (_drawArrow != v) {
	_drawArrow = v;
	update();
    }
}

void PanelButtonBase::slotSetPopupDirection(Direction d)
{
    _popupDirection = d;

    switch (_popupDirection) {
    case ::dUp:    setArrowDirection(::Top);    break;
    case ::dDown:  setArrowDirection(::Bottom); break;
    case ::dLeft:  setArrowDirection(::Left);   break;
    case ::dRight: setArrowDirection(::Right);  break;
    }
}

void PanelButtonBase::setArrowDirection(Position dir)
{
    if (_arrowDirection != dir) {
	_arrowDirection = dir;
	_bgDirty = true;
	update();
    }
}

void PanelButtonBase::setBackground()
{
    //Now here's the trick: whenever a setBackground request comes in
    //and ContainerArea is transparent we have to change our background
    //pixmap because the button or the whole panel have been moved!
    Panel* pContainer = Panel::the();
    ContainerArea *area = pContainer->containerArea();
    if ( _wasTransparent != area->transparent() )
        _bgDirty = true;
    _wasTransparent = area->transparent();
    if (!_bgDirty && !area->transparent() )
        return;

    KConfig* config = KGlobal::config();
    config->setGroup("General");
    if (config->readBoolEntry("UseBackgroundTheme", false) ||
        area->transparent() )
      {
	// Get the pixmap from the container area object
	QPalette pal = pContainer->palette();
	QBrush bgBrush = pal.brush(QPalette::Active, QColorGroup::Background);
	QPixmap* containerBG = bgBrush.pixmap();
        if ( !containerBG && area->transparent() ) {
            containerBG = (QPixmap*) area->paletteBackgroundPixmap();
        }
	// Make sure the background pixmap exists
	if ( containerBG && parent() ) //parent() is null for ZoomButtons
	{
	    // Create a pixmap the same size as the button to use as the bg
	    QPixmap bgPix( width(), height() );

	    // Calculate which part of the container area background should be copied
	    QWidget* p = (QWidget*) parent();

	    int srcx, srcy;
	    if ( _arrowDirection == ::Bottom || _arrowDirection == ::Top )
	    {
		srcx = p->x() % containerBG->width();
		srcy = 0;
	    }
	    else
	    {
		srcx = 0;
		srcy = p->y() % containerBG->height();
	    }

	    // Check if we need to take care of a wrap-around
	    if ( srcx + p->width() <= containerBG->width() &&
		    srcy + p->height() <= containerBG->height() )
	    {
		// Nothing funny going on with overlaps - just copy

		QPoint dp( 0, 0 ); // destination point
		QRect sr( srcx, srcy, width(), height() );
		bitBlt( &bgPix, dp, containerBG, sr, CopyROP );
	    }
	    else
	    {
		// Need to do 2 seperate bitBlts to take care of the overlap

		// Copy the left/top of the image first up to the wrap-
		// around point
		int x = 0;
		int y = 0;
		int w = containerBG->width() - srcx;
		int h = containerBG->height() - srcy;
		QPoint dp( x, y ); // destination point
		QRect sr( srcx, srcy, w, h );
		bitBlt( &bgPix, dp, containerBG, sr, CopyROP );

		// Now copy the wrap-around bit
		if ( _arrowDirection == ::Bottom || _arrowDirection == ::Top )
		{
		    x = containerBG->width() - srcx;
		    y = 0;
		    w = srcx + p->width() - containerBG->width();
		    h = p->height();
		}
		else
		{
		    x = 0;
		    y = containerBG->height() - srcy;
		    w = p->width();
		    h = srcy + p->height() - containerBG->height();
		}
		dp = QPoint( x, y );
		sr = QRect( 0, 0, w, h );
		bitBlt( &bgPix, dp, containerBG, sr, CopyROP );
	    }

	    _bg = bgPix;
	}
	else
	{
	    // Conatainer palette pixmap not set yet
	    _bg = QPixmap();
	}
      }
    else
    {
	// No background pixmap to use
	_bg = (backgroundPixmap() && backgroundPixmap()->size()==size()) ? *backgroundPixmap() :  QPixmap();
    }

    _bgDirty = false;
}

void PanelButtonBase::resizeEvent(QResizeEvent*)
{
    // optimize: reload only when size really changes
    loadTiles();
    _bgDirty = true;
    if(calculateIconSizes())
        loadIcons();
}

void PanelButtonBase::setZoomEnabled(bool b)
{
    if (!zoomButton)
        zoomButton = new ZoomButton();

    if (!b)
      zoomDisabled++;
    else
      zoomDisabled--;

    if (zoomDisabled)
    {
        zoomButton->unFocus();
    }
}

void PanelButtonBase::enterEvent(QEvent* e)
{

    if (!zoomDisabled)
    {
        if (!zoomButton)
            zoomButton = new ZoomButton;

        if (zoomButton->isZoomingEnabled() && /* _icon.width() < 32 && */
          !_iconz.isNull() && _iconz.width() > _icon.width() &&
          !mouseGrabber() && !qApp->activePopupWidget() )
        { // we can and should zoom
            if ( !zoomButton->isWatching( this ) ) {
                zoomButton->watchMe( this );
                update();
            }
            return;
        }
    }

    if (zoomButton->isAnimationEnabled() && _hasAnimation)
    {
        QMovie movie = KGlobal::iconLoader()->loadMovie(_iconName, KIcon::Panel, _size);
        if( !movie.isNull()) {
            delete _movie;
            _movie = new QMovie( movie );
            _movie->connectUpdate( this, SLOT( slotMovieUpdate(const QRect&)));
            _movie->connectStatus( this, SLOT( slotMovieStatus(int)));
            _animated = true;
	    return;
        }
        else {
	    _hasAnimation = false;
        }
    }

    _highlight = true;
    repaint(false);
    QButton::enterEvent(e);
}

void PanelButtonBase::leaveEvent(QEvent* e)
{

    if (_animated) {
        //  Should always disconnect before deletion,
	// because QMovie uses explicit sharing
	_movie->disconnectUpdate( this, SLOT( slotMovieUpdate(const QRect&)));
        _movie->disconnectStatus( this, SLOT( slotMovieStatus(int)));
    	delete _movie;
	_movie = 0L;
	_animated = false;
	repaint(false);
    }

    if (_highlight) {
	_highlight = false;
	repaint(false);
    }
    QButton::leaveEvent(e);
}

void PanelButtonBase::dragEnterEvent(QDragEnterEvent* e)
{
    if (e->isAccepted())
	_hasAcceptedDrag = true;
    update();
    QButton::dragEnterEvent( e );
}

void PanelButtonBase::dragLeaveEvent(QDragLeaveEvent* e)
{
    _hasAcceptedDrag = false;
    update();
    QButton::dragLeaveEvent( e );
}

void PanelButtonBase::dropEvent(QDropEvent* e)
{
    _hasAcceptedDrag = false;
    update();
    QButton::dropEvent( e );
}

void PanelButtonBase::loadTiles()
{
    if (_tile.isNull())
	_up = _down = QPixmap();
    else {
	// If only the tiles were named a bit smarter we wouldn't have
	// to pass the up or down argument.
	_up   = QPixmap(loadTile(_tile, size(), "up"));
	_down = QPixmap(loadTile(_tile, size(), "down"));
    }
}

void PanelButtonBase::drawButton(QPainter *p)
{
    setBackground();

    // Draw the background. This is always needed, even when using tiles,
    // because they don't have to cover the entire button.
    if (!_bg.isNull())
        p->drawPixmap(0, 0, _bg);
    else
        p->fillRect(rect(), colorGroup().brush(QColorGroup::Background));

    const QPixmap& tile = (isDown() || isOn()) ? _down : _up;
    if (!tile.isNull()) {
	// Draw the tile.
	p->drawPixmap(0, 0, tile);
    }
    else if (isDown() || isOn()) {
	// Draw shapes to indicate the down state.
	p->setPen(Qt::black);
	p->drawLine(0, 0, width()-1, 0);
	p->drawLine(0, 0, 0, height()-1);
	p->setPen(colorGroup().light());
	p->drawLine(0, height()-1, width()-1, height()-1);
	p->drawLine(width()-1, 0, width()-1, height()-1);
    }

    drawButtonLabel(p);

    if (hasFocus() || _hasAcceptedDrag) {
	int x1, y1, x2, y2;
	rect().coords(&x1, &y1, &x2, &y2);
	QRect r(x1+2, y1+2, x2-x1-3, y2-y1-3);
	style().drawPrimitive(QStyle::PE_FocusRect, p, r, colorGroup(),
	                      QStyle::Style_Default, colorGroup().button());
    }
}

void PanelButtonBase::drawButtonLabel(QPainter *p)
{
    if (isDown() || isOn())
	p->translate(2,2);

    const QPixmap& icon = labelIcon();
    if (!icon.isNull()
     && !(zoomButton && zoomButton->isWatching(this))) {
	int x = (width()  - icon.width() )/2;
	int y = (height() - icon.height())/2;
	p->drawPixmap(x, y, icon);
    }

    if (_drawArrow) {
	QStyle::PrimitiveElement e = QStyle::PE_ArrowUp;
	QRect r(0, 0, 8, 8);

	switch (_arrowDirection) {
	case ::Top:
	    e = QStyle::PE_ArrowUp;
	    break;
	case ::Bottom:
	    e = QStyle::PE_ArrowDown;
	    r.moveBy(0, height() - 8);
	    break;
	case ::Right:
	    e = QStyle::PE_ArrowRight;
	    r.moveBy(width() - 8 , 0);
	    break;
	case ::Left:
	    e = QStyle::PE_ArrowLeft;
	    break;
	}

	int flags = QStyle::Style_Enabled;
	if ( isDown() || isOn() )
	    flags |= QStyle::Style_Down;
	style().drawPrimitive(e, p, r, colorGroup(), flags);
    }

    if (isDown() || isOn())
	p->translate(-2,-2);
}

const QPixmap& PanelButtonBase::labelIcon() const
{
    if (_animated)
	return _movie->framePixmap();
    else
	return _highlight ? _iconh : _icon;
}

void PanelButtonBase::slotMovieUpdate(const QRect& /*rect*/)
{
    if (_animated) {
	emit iconChanged();
	update(); // update all since rect seems to be set incorrect
	// update(rect);
    }
}

void PanelButtonBase::slotMovieStatus( int status )
{
    if (status < 0) {
        // Error playing the MNG -> forget about it and do normal iconeffect
        if (_animated) {
            delete _movie;
            _movie = 0;
            _animated =  false ;
            _hasAnimation = false;
	    emit iconChanged();
	    update();
        }
    }
    else if (status == QMovie::EndOfMovie) {
	// We could trigger on end of movie here, we don't because we wish to
	// view the last frame of the movie for the rest of the mouse-over .
    }
}

QImage loadTile(const QString& tile, const QSize& size, const QString& state)
{
    QString name = tile;

    if (size.height() < 42)
	name += "_tiny_";
    else if (size.height() < 54)
	name += "_normal_";
    else
	name += "_large_";

    name += state + ".png";

    QImage tileImg(KGlobal::dirs()->findResource("tiles", name));

    // scale if size does not match exactly
    if (!tileImg.isNull() && tileImg.size() != size)
	tileImg = tileImg.smoothScale(size);

    return tileImg;
}


